<!-- home page for vanilla application -->

{% extends 'blti/base.html' %}

{% block content %}

<div class="clearfix">

<h2>BLTI Launch Parameters</h3>

<div>
  <h3>Launch View Model</h3>
  <p>Parameters passed to this tool by the platform after being digested by the LTI library:</p>
  <table>
  {% for key, value in digested_lti_params %}
    <tr>
      <td><strong>{{ key }}</strong></td><td>{{ value }}</td>
    </tr>
  {% endfor %}
  </table>
</div>

<div>
  <h3>Raw Launch Data</h3>
  <p>Here are the parameters that were passed to this tool by the platform:</p>
  <pre>
{{ raw_lti_params }}
  </pre>
</div>

</div>

{% endblock %}
